/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CIRCULARBUFFER_H
#define CIRCULARBUFFER_H

#include <cstddef>

template <class T, std::size_t SIZE>
class CircularBuffer
{
public:
	CircularBuffer();
	T& next();
	T& operator[](std::size_t i);
private:
	T data[SIZE];
	std::size_t current;
};

template <class T, std::size_t SIZE>
CircularBuffer<T, SIZE>::CircularBuffer()
:	current(0)
{
}

template <class T, std::size_t SIZE>
T& CircularBuffer<T, SIZE>::next()
{
	current = (current + 1) % SIZE;
	return data[current];
}
template <class T, std::size_t SIZE>
T& CircularBuffer<T, SIZE>::operator[](std::size_t i)
{
	return data[(SIZE + current - i) % SIZE];
}

#endif
