/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ComputerPlayerSettingsMenu.h"
#include "Settings.h"
#include "StateManager.h"
#include "SettingsMenu.h"

ComputerPlayerSettingsMenu::ComputerPlayerSettingsMenu()
:	player(1),
	save(false),
	cancel(false),
	difficulty(-1)
{
	menu.setTitle("Computer Player Settings");
	
	menu.addItem("Player", &player, 1, 8);
	menu.addItem("Name", &name);
	menu.addItem("Difficulty", &difficulty, -1, 4, "default", "very easy", "easy", "normal", "hard", "very hard");
	menu.addItem("Save settings", &save);
	menu.addItem("Cancel", &cancel);
	
	menu.addRenderObjects(this);
	
	for (int i = 0; i < 8; ++i)
	{
		names[i] = Settings::players.computerNames[i];
		difficulties[i] = Settings::players.computerDifficulties[i];
	}
	updatePlayer();
}

void ComputerPlayerSettingsMenu::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	MenuEvent me = StateManager::getInstance().getMenuEvent(event);
	
	int oldPlayer = player;
	
	if (menu.onEvent(me))
	{
		if (save)
		{
			for (int i = 0; i < 8; ++i)
			{
				Settings::players.computerNames[i] = names[i];
				Settings::players.computerDifficulties[i] = difficulties[i];
			}
			sm.enter(new SettingsMenu(), -1);
		}
		else if (cancel)
		{
			sm.enter(new SettingsMenu(), -1);
		}
		else if (player != oldPlayer)
		{
			updatePlayer();
		}
	}
	else if (me.type == MENU_ESC)
	{
		sm.enter(new SettingsMenu(), -1);
	}
}

void ComputerPlayerSettingsMenu::updatePlayer()
{
	int i = player - 1;
	name = names[i];
	menu.notifyValueUpdate(&name);
	difficulty = difficulties[i];
	menu.notifyValueUpdate(&difficulty);
}

void ComputerPlayerSettingsMenu::onValueUpdate(std::string* strValue)
{
	if (strValue == &name)
	{
		names[player - 1] = name;
	}
}

void ComputerPlayerSettingsMenu::onValueUpdate(int* intValue)
{
	if (intValue == &difficulty)
	{
		difficulties[player - 1] = difficulty;
	}
}
