/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ConnectToNetwork.h"
#include "StateManager.h"
#include "NetworkMenu.h"
#include "MainMenu.h"
#include "Internet.h"
#include "LAN.h"
#include "ErrorState.h"

ConnectToNetwork::ConnectToNetwork()
:	connectAction(false),
	cancelAction(false)
{
	// Disconnect possible existing network.
	StateManager::getInstance().setNetwork(0);
	
	menu.setTitle("Connect to network");
	
	menu.addItem("Players", &Settings::network.players, 1, 7);
	menu.addItem("Network", &Settings::network.network, 0, 1, "Internet", "LAN");
	menu.addItem("Connect", &connectAction);
	menu.addItem("Cancel", &cancelAction);
	
	menu.addRenderObjects(this);
}

void ConnectToNetwork::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	MenuEvent me = sm.getMenuEvent(event);
	if (menu.onEvent(me))
	{
		if (connectAction)
		{
			Network* net;
			if (Settings::network.network)
			{
				net = new LAN;
			}
			else
			{
				net = new Internet;
			}
			
			std::string error = net->getError();
			if (error.empty())
			{
				sm.setNetwork(net);
				sm.enter(new NetworkMenu(net), +1);
			}
			else
			{
				delete net;
				sm.enter(new ErrorState("Network error", error, new ConnectToNetwork()), +1);
			}
			return;
		}
	}
	else if (me.type == MENU_ESC)
	{
		cancelAction = true;
	}
	if (cancelAction)
	{
		sm.enter(new MainMenu(), -1);
	}
}
