/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CreateNetworkGame.h"
#include "Network.h"
#include "NetworkMenu.h"
#include "StateManager.h"
#include "Settings.h"
#include "CarSelectionState.h"

void makeSureGameNameNotEmpty()
{
	if (Settings::network.name.empty())
	{
		Settings::network.name = Settings::players.humanNames[0] + "'s game";
	}
}

CreateNetworkGame::CreateNetworkGame(Network* net)
:	net(net),
	raceTypeUndecided(true),
	singleRace(false),
	tournament(false),
	ok(false)
{
	makeSureGameNameNotEmpty();
	
	menu.setTitle("Create " + net->name() + " game");
	
	menu.addItem("Single Race", &singleRace);
	menu.addItem("Tournament", &tournament);
	menu.addItem("Cancel", &cancel);
	
	menu.addRenderObjects(this);
	net->setGameState(this, false);
}

void CreateNetworkGame::update()
{
	net->step();
	menu.update();
}

void CreateNetworkGame::onEvent(const SDL_Event& event)
{
	if (!net->onEvent(event))
	{
		StateManager& sm = StateManager::getInstance();
		MenuEvent me = sm.getMenuEvent(event);
		if (menu.onEvent(me))
		{
			if (raceTypeUndecided && (singleRace || tournament))
			{
				Settings::network.tournament = tournament;
				menu.reset();
				menu.addItem("Name", &Settings::network.name, 25);
				menu.addItem("Max players", &Settings::network.maxPlayers, 2, 8);
				menu.addItem("Max players per client", &Settings::network.maxPlayersPerClient, 1, 7);
				
				int maxComputerPlayers = 7 - Settings::network.players;
				if (Settings::network.computerPlayers > maxComputerPlayers)
				{
					Settings::network.computerPlayers = maxComputerPlayers;
				}
				menu.addItem("Computer players", &Settings::network.computerPlayers, 0, maxComputerPlayers);
				menu.addItem("Computer difficulty", &Settings::network.computerDifficulty, 0, 4, "very easy", "easy", "normal", "hard", "very hard");
				menu.addItem("Laps", &Settings::network.laps, 1, 9);
				if (tournament)
				{
					menu.addItem("Tracks", &Settings::network.tracks, 2, 99);
				}
				menu.addItem("Car collisions", &Settings::network.collisions, 0, 1, "off", "on");
				menu.addItem("Create", &ok);
				menu.addItem("Cancel", &cancel);
				menu.addRenderObjects(this);
				raceTypeUndecided = false;
				
				// This is needed to make mouse click select the correct item.
				setStateInfo(0);
			}
			else if (ok)
			{
				makeSureGameNameNotEmpty();
				net->setGame(Settings::network.name, Settings::network.maxPlayers, Settings::network.maxPlayersPerClient, Settings::network.laps, (tournament? Settings::network.tracks : 1), Settings::network.collisions);
				sm.enter(new CarSelectionState(CAR_SELECTION_NETWORK_CREATE, net), 0);
				return;
			}
		}
		else if (me.type == MENU_ESC)
		{
			cancel = true;
		}
		
		if (cancel)
		{
			sm.enter(new NetworkMenu(net), -1);
		}
	}
}
