/*
 * Copyright 2014, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DCOLOR_H
#define DCOLOR_H

#include "SDL.h"
#include <cstddef>

struct DColor
{
	double r;
	double g;
	double b;
};

struct Scale32
{
	typedef Uint32 PixelType;
	static const std::size_t  pixelSize = 1;
	static inline void readColors(const Uint32* src, DColor& c)
	{
		Uint32 p = *(src);
		c.r = (p & 0xFF0000) >> 16;
		c.g = (p & 0x00FF00) >> 8;
		c.b = (p & 0x0000FF);
	}
	static inline void writeColors(Uint32* dst, const DColor& c)
	{
		*dst = (int(c.r) << 16) | (int(c.g) << 8) | int(c.b);
	}
};

struct Scale24
{
	typedef Uint8 PixelType;
	static const std::size_t pixelSize = 3;
	static inline void readColors(const Uint8* src, DColor& c)
	{
		c.r = src[0];
		c.g = src[1];
		c.b = src[2];
	}

	static inline void writeColors(Uint8* dst, const DColor& c)
	{
		dst[0] = c.r;
		dst[1] = c.g;
		dst[2] = c.b;
	}
};

#endif
