/*
 * Copyright 2010, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DRIVER_H
#define DRIVER_H

#include "Track.h"
#include "SDL.h"
#include <ostream>
#include <istream>

const int DRIVER_ACCELERATE = 0x01;
const int DRIVER_BRAKE      = 0x02;
const int DRIVER_LEFT       = 0x04;
const int DRIVER_RIGHT      = 0x08;

const int CONTROL_ACCELERATE = 0;
const int CONTROL_BRAKE      = 1;
const int CONTROL_LEFT       = 2;
const int CONTROL_RIGHT      = 3;

class Driver
{
public:
	Driver() : deviceIndex(-1) {}
	virtual ~Driver(){}
	virtual int drive(int carId, const Track& track) = 0;
	virtual void save(std::ostream&) const {}
	virtual void load(std::istream&) {}
	virtual bool setControl(const SDL_Event& event, int control) { return false; }
	virtual std::string getControlName(int control) const { return "-"; }
	virtual bool isAI() const { return false; }
	
	void setDeviceIndex(int i) { deviceIndex = i; }
	int getDeviceIndex() const { return deviceIndex; }
private:
	int deviceIndex;
};

#endif
