/*
 * Copyright 2010, 2012, 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Font.h"
#include "Path.h"
#include <fstream>

Font::Font(const std::string& file)
{
	Surface fontSurface("fonts/" + file + ".png");
	fontHeight = fontSurface.height();
	
	std::ifstream fontFile((dataPath() + "fonts" + (file + ".font")).c_str());
	
	int x = 0;
	char c;
	int charWidth;
	
	fontFile >> charWidth;
	
	if (fontFile.good())
	{
		charMap.insert(std::make_pair(' ', fontSurface.subSurface(0, 0, charWidth, fontHeight)));
		x += charWidth;
	}

	while (fontFile >> c >> charWidth && (x + charWidth) <= fontSurface.width())
	{
		charMap.insert(std::make_pair(c, fontSurface.subSurface(x, 0, charWidth, fontHeight)));
		x += charWidth;
	}
}



Surface Font::createTextSurface(const std::string& str, Color c) const
{
	if (c == 0xFFFFFF)
	{
		c = 0xFEFEFE;
	}

	int width = calculateTextWidth(str);
	
	Surface textSurface(width, fontHeight);
	
	width = 0;
	for (std::string::const_iterator it = str.begin(); it != str.end(); ++it)
	{
		std::map<char, Surface>::const_iterator mit = charMap.find(std::tolower(*it));
		if (mit != charMap.end())
		{
			mit->second.draw(textSurface, width, 0);
			width += mit->second.width();
		}
	}
	textSurface.setColorKey(0x000000);
	Surface textSurface2(width, fontHeight);
	textSurface2.setColorKey(0xFFFFFF);
	textSurface2.fill(c);
	textSurface.draw(textSurface2, 0, 0);
	
	return textSurface2;
}


Surface Font::createTextSurface(const std::string& str, Color c1, Color c2) const
{
	Surface textSurface = createTextSurface(str, c1);
	Surface borderSurface = createTextSurface(str, c2);
	Surface finalSurface(textSurface.width() + 2, textSurface.height() + 2);
	Color colorKey = 0;
	while (colorKey == c1 || colorKey == c2)
	{
		++colorKey;
	}
	finalSurface.fill(colorKey);
	finalSurface.setColorKey(colorKey);
	for (int y = 0; y < 3; ++y)
	{
		for (int x = 0; x < 3; ++x)
		{
			borderSurface.draw(finalSurface, x, y);
		}
	}
	textSurface.draw(finalSurface, 1, 1);	
	
	return finalSurface;
}

int Font::calculateTextWidth(const std::string& str) const
{
	int width = 0;
	for (std::string::const_iterator it = str.begin(); it != str.end(); ++it)
	{
		std::map<char, Surface>::const_iterator mit = charMap.find(std::tolower(*it));
		if (mit != charMap.end())
		{
			width += mit->second.width();
		}
	}
	return width;
}

