/*
 * Copyright 2010, 2011, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "GameState.h"
#include "StateManager.h"
#include <cassert>
#include <algorithm>

GameState::GameState()
:	nextUpdate(SDL_GetTicks())
{
	background = StateManager::getInstance().getBackground();
}

GameState::~GameState()
{
}

void GameState::update()
{
	menu.update();
}

const Uint32 TIME_STEP = 20;

int GameState::sleepTime()
{
	Uint32 now = SDL_GetTicks();
	nextUpdate += TIME_STEP;
	if (nextUpdate <= now)
	{
		// This ensures the timing will not fall behind and 
		// that we always sleep at least one millisecond.
		nextUpdate = now + 1;
	}
	return nextUpdate - now;
}

void GameState::onEvent(const SDL_Event& event)
{
}

void GameState::addRenderObject(RenderObject& ro, int z)
{
	assert(std::find(renderObjects.begin(), renderObjects.end(), &ro) == renderObjects.end());
	
	ro.flagUpdated();
	
	for (std::size_t i = 0; i < renderObjects.size(); i++)
	{
		if (renderObjectsZ[i] > z)
		{
			renderObjects.insert(renderObjects.begin() + i, &ro);
			renderObjectsZ.insert(renderObjectsZ.begin() + i, z);
			return;
		}
	}
	// if we know the object should be added to the end. 
	renderObjects.push_back(&ro);
	renderObjectsZ.push_back(z);
}

void GameState::removeRenderObject(RenderObject& ro)
{
	for (std::size_t i = 0; i < renderObjects.size(); i++)
	{
		if (renderObjects[i] == &ro)
		{
			renderObjects.erase(renderObjects.begin() + i);
			renderObjectsZ.erase(renderObjectsZ.begin() + i);
			Rect rect;
			ro.getOldRect(rect);
			StateManager::getInstance().addUpdateRect(rect);
			break;
		}
	}
}

int GameState::getStateInfo()
{
	return menu.getItem();
}

void GameState::setStateInfo(int selectedItem)
{
	menu.setItem(selectedItem);
	
	const MenuEvent& e = StateManager::getInstance().getLastMenuEvent();
	if  (e.type == MENU_MOUSE_CLICK1 ||
	     e.type == MENU_MOUSE_CLICK2)
	{
		menu.makeActive(ITEM_UNSELECTED);
		menu.mouseMove(e.x, e.y);
	}
}

void GameState::validateTypedValue(const std::string* strValue, std::string& inputString)
{
}

void GameState::onValueUpdate(int* intValue)
{
}

void GameState::onValueUpdate(std::string* strValue)
{
}
