/*
 * Copyright 2010, 2011, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GAMESTATE_H
#define GAMESTATE_H

#include "Surface.h"
#include "Menu.h"
#include <vector>


class RenderObject;


class GameState
{
public:
	GameState();
	virtual ~GameState();
public:
	virtual void update();
	virtual int sleepTime();
	virtual void onEvent(const SDL_Event& event);
	virtual int getStateInfo();
	virtual void setStateInfo(int info);
	virtual void validateTypedValue(const std::string* strValue, std::string& inputString);
	virtual void onValueUpdate(int* intValue);
	virtual void onValueUpdate(std::string* strValue);

	void addRenderObject(RenderObject& ro, int z = 0);
	void removeRenderObject(RenderObject& ro);
protected:
	Surface background;
	Menu menu;
private:
	Uint32 nextUpdate;
	std::vector<RenderObject*> renderObjects;
	std::vector<int> renderObjectsZ;
	
	friend class StateManager;
	friend void Menu::addRenderObjects(GameState*, int, int, int, int);
};

#endif



