/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Ghost.h"
#include "BinaryFile.h"
#include "GameState.h"
#include "SurfaceBank.h"

Ghost::Ghost(int n)
:	iState(0),
	gameState()
{
	renderObject.setSurface(SurfaceBank::getCar(8 + n));
}

void Ghost::reset(const std::vector<GhostState>& newData, GameState& state)
{
	gameState = &state;
	iState = 0;
	states = newData;
	if (!states.empty())
	{
		renderObject.update(-100, 0, 0);
		gameState->addRenderObject(renderObject);
	}
}

void Ghost::disable()
{
	iState = 0;
	gameState = 0;
	states.clear();
}

void Ghost::update()
{
	if (iState < states.size())
	{
		const GhostState& state = states[iState];
		renderObject.update(state.x, state.y, state.rotation);
		++iState;
	}
	else if (gameState)
	{
		gameState->removeRenderObject(renderObject);
		gameState = 0;
	}
}
