/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "JoinNetworkGameFinal.h"
#include "Network.h"
#include "Settings.h"
#include "JoinNetworkGame.h"
#include "StateManager.h"
#include "NetworkMenu.h"
#include "str.h"

JoinNetworkGameFinal::JoinNetworkGameFinal(Network* net, int joinedPlayers)
:	net(net),
	descriptionText(70),
	join(false),
	cancel(false),
	timeoutText(160)
{
	net->setGameState(this, false);
	
	menu.setTitle("Join game?");
	
	descriptionText.setText("The following players could not join:");
	descriptionText.addTo(this);
	
	int playersLeft = Settings::network.players - joinedPlayers;
	playerTexts.reserve(playersLeft);
	for (int i = 0; i < playersLeft; ++i)
	{
		MenuText mt(Settings::players.humanNames[joinedPlayers + i], 82 + 12 * i);
		playerTexts.push_back(mt);
		playerTexts.back().addTo(this);
	}
	
	menu.addItem("Join anyway", &join);
	menu.addItem("Cancel", &cancel);
	
	menu.addRenderObjects(this, CENTER_X, 100 + playersLeft * 12);
	
	timeoutStart = SDL_GetTicks();
	if (playersLeft > 3)
	{
		timeoutText = MenuText(120 + playersLeft * 12);
	}
	timeoutText.addTo(this);
	timeoutSeconds = 0;
}

void JoinNetworkGameFinal::update()
{
	menu.update();
	net->step();
	
	int newTimeoutSeconds = (timeoutStart + 16000 - static_cast<int>(SDL_GetTicks())) / 1000;
		
	if (newTimeoutSeconds != timeoutSeconds)
	{
		timeoutSeconds = newTimeoutSeconds;
		if (timeoutSeconds <= 0)
		{
			join = true;
		}
		else
		{
			timeoutText.setText("The game will be joined automatically in " + int2string(timeoutSeconds) + " seconds");
		}
	}
	
	if (join)
	{
		StateManager::getInstance().enter(new JoinNetworkGame(net, -1), 0);
	}
	else if (cancel)
	{
		net->quitGame();
		StateManager::getInstance().enter(new NetworkMenu(net), -1);
	}
}

void JoinNetworkGameFinal::onEvent(const SDL_Event& event)
{
	menu.onEvent(event);
}
