/*
 * Copyright 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "JoystickDevice.h"
#include "JoystickDriver.h"
#include <sstream>

JoystickDevice::JoystickDevice(int i)
{
	const char* name = SDL_JoystickName(i);
	if (name)
	{
		// Remove repeated words to make the name shorter.
		std::istringstream iss(name);
		std::string word;
		std::string prevWord;
		while (iss >> word)
		{
			if (word != prevWord)
			{
				if (!this->name.empty())
				{
					this->name += ' ';
				}
				this->name += word;
				word.swap(prevWord);
			}
		}
	}
	
	if (this->name.empty())
	{
		std::ostringstream oss;
		oss << "Unknown device (" << i << ")";
		this->name = oss.str();
	}
	
	joy = SDL_JoystickOpen(i);
}

JoystickDevice::~JoystickDevice()
{
	if (joy)
	{
		SDL_JoystickClose(joy);
	}
}

std::string JoystickDevice::getName() const
{
	return name;
}

Driver* JoystickDevice::createDriver() const
{
	return new JoystickDriver(name, joy);
}

