/*
 * Copyright 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "KeyboardDriver.h"
#include <limits>

KeyboardDriver::KeyboardDriver()
{
	for (int k = 0; k < 4; ++k)
	{
		keys[k] = SDLK_UNKNOWN;
	}
}

KeyboardDriver::KeyboardDriver(SDLKey accKey, SDLKey brakeKey, SDLKey leftKey, SDLKey rightKey)
{
	keys[CONTROL_ACCELERATE] = accKey;
	keys[CONTROL_BRAKE] = brakeKey;
	keys[CONTROL_LEFT] = leftKey;
	keys[CONTROL_RIGHT] = rightKey;
}

int KeyboardDriver::drive(int, const Track&)
{	
	int flags = 0;
	Uint8* keystate = SDL_GetKeyState(0);
	for (int k = 0; k < 4; ++k)
	{
		if (keystate[keys[k]])
		{
			flags |= 1 << k;
		}
	}

	return flags;
}

bool KeyboardDriver::setControl(const SDL_Event& event, int control)
{
	if (event.type == SDL_KEYDOWN)
	{
		switch (event.key.keysym.sym)
		{
		case SDLK_TAB:
		case SDLK_F11:
		case SDLK_ESCAPE:
			return false;
		default:
			keys[control] = event.key.keysym.sym;
			return true;
		}
	}
	return false;
}

std::string KeyboardDriver::getControlName(int k) const
{
	if (keys[k] == SDLK_UNKNOWN)
	{
		return "";
	}
	return SDL_GetKeyName(static_cast<SDLKey>(keys[k]));
}

void KeyboardDriver::save(std::ostream& out) const
{
	out << "Keyboard\n";
	for (int k = 0; k < 4; ++k)
	{
		out << ' ' << keys[k];
	}
	out << '\n';
}

void KeyboardDriver::load(std::istream& in)
{
	for (int k = 0; k < 4; ++k)
	{
		in >> keys[k];
	}
	in.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
}
