/*
 * Copyright 2010, 2011, 2012, 2013. 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MainMenu.h"
#include "StateManager.h"
#include "SettingsMenu.h"
#include "ConnectToNetwork.h"
#include "RacePreparationMenu.h"
#include "TimeTrialMenu.h"
#include <cmath>

MainMenu::MainMenu()
:	singleRaceAction(false),
	tournamentAction(false),
	networkAction(false),
	settingsAction(false),
	quitAction(false)
{
	if (!Settings::timetrial.remember)
	{
		// The time trial driver name is forgotten 
		// to avoid that someone uses it by accident.
		Settings::timetrial.name.clear();
	}
	
	menu.setTitle("Brum Brum Rally");
	
	menu.addItem("Single race", &singleRaceAction);
	menu.addItem("Tournament", &tournamentAction);
	menu.addItem("Time trial", &timeTrialAction);
	menu.addItem("Network", &networkAction);
	menu.addItem("Settings", &settingsAction);
	menu.addItem("Quit", &quitAction);
	
	menu.addRenderObjects(this);
}

void MainMenu::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	if (menu.onEvent(event))
	{
		if (singleRaceAction)
		{
			sm.enter(new RacePreparationMenu(false), +1);
		}
		else if (tournamentAction)
		{
			sm.enter(new RacePreparationMenu(true), +1);
		}
		else if (timeTrialAction)
		{
			sm.enter(new TimeTrialMenu(), +1);
		}
		else if (networkAction)
		{
			sm.enter(new ConnectToNetwork(), +1);
		}
		else if (settingsAction)
		{
			sm.enter(new SettingsMenu(), +1);
		}
		else if (quitAction)
		{
			sm.quit();
		}
	}
}
