/*
 * Copyright 2011, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MENU_H
#define MENU_H

#include "RenderObject.h"
#include "Vector.h"
#include <vector>

class GameState;
class GameState;

enum
{
	MENU_UNKNOWN,
	MENU_ACTION,
	MENU_NEXT,
	MENU_PREV,
	MENU_INC,
	MENU_DEC,
	MENU_ESC,
	MENU_FIRST,
	MENU_LAST,
	MENU_MIN,
	MENU_MAX,
	MENU_MOUSE_MOVE,
	MENU_MOUSE_CLICK1,
	MENU_MOUSE_CLICK2,
	MENU_MOUSE_RELEASE,
	MENU_TEXT,
	MENU_ERASE,
	MENU_DELETE
};


struct MenuEvent
{
	int type;
	short x;
	short y;
	char c;
};

enum ItemState
{
	ITEM_UNSELECTED,
	ITEM_SELECTED,
	ITEM_ACTIVE
};

const int CENTER_X = 160;
const int CENTER_Y = 140;

class Menu
{
public:
	Menu();
	
	void addItem(const std::string& label, bool* fireAction = 0);
	void addItem(const std::string& label, int* value, int min, int max, const char* firstStrValue = 0, ...);
	void addItem(const std::string& label, int* value, const std::vector<std::string>& strValues);
	void addItem(const std::string& label, std::string* value, int max = 12);
	void addItem(const std::string& label, std::string* value, int max, const std::vector<std::string>& strValues);
	void appendSuffix(const std::string& suffix);
	void setSuffix(int* value, const std::string& suffix);
	
	void addRenderObjects(GameState* gs, int x = CENTER_X, int y = CENTER_Y, int dx = 0, int dy = 11);
	
	bool onEvent(const SDL_Event& event);
	bool onEvent(MenuEvent event);
	
	void next();
	void prev();
	void action();
	void inc();
	void dec();
	bool escape();
	bool del();
	void first();
	void last();
	void min();
	void max();
	void mouseMove(int x, int y);
	void click(int button, int x, int y);
	void release();
	void input(char c);
	void erase();
	
	void notifyValueUpdate(int* value);
	void notifyValueUpdate(std::string* value);
	bool activeItemValue(int* value) const;
	void makeActive(ItemState state = ITEM_ACTIVE);
	void setLabel(int i, const std::string& label);
	
	void reset();
	
	void setTitle(const std::string& newTitle);
	void update();
	
	std::size_t getItem() const;
	void setItem(std::size_t i);
	
	bool isActive() const;
	ItemState getItemState() const;
	int getItem(int x, int y);
	
	bool isVisible() const;
	void hide();
	void show(GameState* gs);
private:
	void updateItem(std::size_t i);
	void applyInput();
	struct Item
	{
		std::string label;
		bool* fireAction;
		int* intValue;
		std::string* strValue;
		int min;
		int max;
		bool wrap;
		std::vector<std::string> strValues;
		RenderObject renderObject;
		std::string suffix;
	};
	std::vector<Item> items;
	std::size_t selectedItem;
	int hoverItem;
	ItemState selectedItemState;
	std::string inputString;
	
	std::string title;
	RenderObject titleRenderObject;
	double titleRotation;
	double titleOldRotation;
	double titleRotationSpeed;
	double titleRotationSpeedLimit;
	bool titleDrag;
	Vector titlePos;
	Vector titleDragPos;
	
	GameState* state;
	
};

#endif
