/*
 * Copyright 2010, 2011, 2012, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "RacePreparationMenu.h"
#include "StateManager.h"
#include "CarSelectionState.h"
#include "MainMenu.h"
#include "Settings.h"
#include <fstream>

const int MAX_PLAYERS = 8;
const int MIN_PLAYERS = 1;
const int MAX_LAPS = 9;
const int MIN_LAPS = 1;
const int MAX_TRACKS = 100;
const int MIN_TRACKS = 1;

RacePreparationMenu::RacePreparationMenu(bool tournament)
:	ok(false),
	cancel(false),
	path(configPath() + (tournament ? "/tournament" : "/single"))
{
	adjustPlayers();
	
	menu.setTitle(tournament ? "tournament" : "single race");
	
	Settings::race.tournament = tournament;
	
	menu.addItem("Human players", &Settings::race.humanPlayers, 0, 8);
	menu.addItem("Computer players", &Settings::race.computerPlayers, 0, 8);
	menu.addItem("Laps", &Settings::race.laps, 1, 9);
	if (tournament)
	{
		menu.addItem("Tracks", &Settings::race.tracks, 1, 100);
	}
	menu.addItem("Car collisions", &Settings::race.collisions, 0, 1, "off", "on");
	menu.addItem("Computer difficulty", &Settings::race.difficulty, 0, 4, "very easy", "easy", "normal", "hard", "very hard");
	menu.addItem("Ok", &ok);
	menu.addItem("Cancel", &cancel);
	
	menu.addRenderObjects(this);
}

void RacePreparationMenu::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	
	MenuEvent me = sm.getMenuEvent(event);
	if (menu.onEvent(me))
	{
		if (ok)
		{
			sm.enter(new CarSelectionState(CAR_SELECTION_NORMAL), +1);
			return;
		}
		
		adjustPlayers();
	}
	else if (me.type == MENU_ESC)
	{
		cancel = true;
	}
	
	if (cancel)
	{
		sm.enter(new MainMenu(), -1);
	}
}

void RacePreparationMenu::adjustPlayers()
{
	if (Settings::race.humanPlayers + Settings::race.computerPlayers > 8)
	{
		if (menu.activeItemValue(&Settings::race.humanPlayers))
		{
			Settings::race.computerPlayers = 8 - Settings::race.humanPlayers;
			menu.notifyValueUpdate(&Settings::race.computerPlayers);
		}
		else
		{
			Settings::race.humanPlayers = 8 - Settings::race.computerPlayers;
			menu.notifyValueUpdate(&Settings::race.humanPlayers);
		}
	}
	else if (Settings::race.humanPlayers + Settings::race.computerPlayers <= 0)
	{
		if (menu.activeItemValue(&Settings::race.humanPlayers))
		{
			Settings::race.computerPlayers = 1;
			menu.notifyValueUpdate(&Settings::race.computerPlayers);
		}
		else
		{
			Settings::race.humanPlayers = 1;
			menu.notifyValueUpdate(&Settings::race.humanPlayers);
		}
	}
}
