/*
 * Copyright 2010, 2011, 2012, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RACESTATE_H
#define RACESTATE_H

#include "GameState.h"
#include "Track.h"
#include "RenderObject.h"
#include "RaceSetup.h"
#include "GameState.h"

class Driver;
class Map;
class Network;

class RaceState : public GameState
{
public:
	RaceState(const Map& map, const RaceSetup& raceSetup, Network* net = 0);
	virtual void update();
	void updateTimeState();
	virtual int sleepTime();
	virtual void onEvent(const SDL_Event& event);
	Track* getTrack();
private:
	void updateLapRO(int i);
	void updatePosRO(int i);
	void updateTimeRO();
	void removePlayers(const std::vector<int>& removedCars);
	void placeStats();
	
	struct Car
	{
		RenderObject carRO;
		RenderObject posRO;
		RenderObject timeRO;
		RenderObject lapRO;
		int pos;
		int lap;
		int time;
	};
	RaceSetup raceSetup;

	std::vector<Car> cars;
	
	//RenderObject renderCars[8];
	Track track;
	Uint32 gameTime;
	Uint32 raceTime;
	
	struct CarProgress
	{
		int carId;
		int progress;
		bool operator<(const RaceState::CarProgress& cp) const;
	};
	CarProgress carProgress[8];
	Surface smallNumbers[10];
	Surface bigNumbers[10];
	Uint32 timeNumbers[6];
	RenderObject timeRO[6];
	RenderObject readyRO;
	RenderObject setRO;
	RenderObject goRO;
	bool endRace;
	int timeState;
	Map map;
	Network* net;
	bool closeMenu;
	bool quit;
	bool restartRace;
	bool newTrack;
	bool selectTrack;
};

const unsigned MAX_RACE_TIME = 3600000;

#endif
