/*
 * Copyright 2014, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SCALER_H
#define SCALER_H

#include "Surface.h"
#include "StateManager.h"

class Scaler
{
public:
	Scaler();
	virtual ~Scaler();
	virtual void update(int w, int h);
	void render(const Surface& surface, SDL_Rect* area);
	
	inline int src2dst_x(int x) const { return x * screenArea.w / GAME_WIDTH + screenArea.x; };
	inline int src2dst_y(int y) const { return y * screenArea.h / GAME_HEIGHT + screenArea.y; };
	inline int dst2src_x(int x) const { return (x - screenArea.x) * GAME_WIDTH / screenArea.w; };
	inline int dst2src_y(int y) const { return (y - screenArea.y) * GAME_HEIGHT / screenArea.h; };
	
	virtual bool isValidZoom(int zoom) const;
	virtual bool isValidBpp(int bpp) const;
	virtual bool needStretchBuffer() const;
	
	void stretchArea(const SDL_Rect& srcArea, SDL_Rect& dstArea, int x, int y, SDL_Surface* source, SDL_Surface* destination);
	void expand(SDL_Rect& srcArea, SDL_Rect& dstArea, int pixels);
	
protected:
	virtual void renderStretch(const SDL_Rect& srcArea, SDL_Rect& dstArea, SDL_Surface* source, SDL_Surface* destination) = 0;
	virtual void renderZoom(const SDL_Rect& srcArea, SDL_Rect& dstArea, SDL_Surface* source, SDL_Surface* destination) = 0;
	
	SDL_Rect screenArea;
	int zoom;
	int xZoom;
	int yZoom;
	
	SDL_Surface* buffer;
	bool stretch;
};

#endif
