/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SelectNewTrack.h"
#include "TileSet.h"
#include "StateManager.h"
#include "Settings.h"
#include "RaceState.h"
#include "SelectTrack.h"
#include "TimeTrialTrack.h"

SelectNewTrack::SelectNewTrack()
:	map(Map::generate()),
	newTrack(false),
	selectTrack(false),
	cancel(false)
{
	TileSet::getTileSet(map.getTilesetName()).decorate(map, background);

	menu.addItem("New track", &newTrack);
	menu.addItem("Laps", &Settings::timetrial.laps, 1, 9);
	menu.addItem("Ok", &selectTrack);
	menu.addItem("Cancel", &cancel);
	
	menu.addRenderObjects(this, CENTER_X, 112);
}

void SelectNewTrack::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	
	MenuEvent me = sm.getMenuEvent(event);
	if (menu.onEvent(me))
	{
		if (newTrack)
		{
			sm.enter(new SelectNewTrack(), 0);
			return;
		}
		else if (selectTrack)
		{
			std::vector<Driver*> drivers;
			std::vector<std::string> names;
			std::vector<int> cars;
			
			drivers.push_back(sm.getDriver(0));
			names.push_back(Settings::timetrial.name);
			cars.push_back(Settings::timetrial.car);
			
			sm.enter(new RaceState(map, RaceSetup(drivers, cars, names, Settings::timetrial.laps, 1, false, false, new TimeTrialTrack(map, Settings::timetrial.laps))), 0);
			return;
		}
	}
	else if (me.type == MENU_ESC)
	{
		cancel = true;
	}
	
	if (cancel)
	{
		sm.enter(new SelectTrack(), -1);
	}
}
