/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "SelectTrack.h"
#include "TimeTrialTrack.h"
#include "StateManager.h"
#include "SelectNewTrack.h"
#include "RaceState.h"
#include "TimeTrialMenu.h"
#include "DeleteTrack.h"

SelectTrack::SelectTrack()
:	trackList(*this)
{
	menu.setTitle("Select track");
	
	menu.addRenderObjects(this);
}

void SelectTrack::onEvent(const SDL_Event& event)
{
	menu.onEvent(event);
	
	StateManager& sm = StateManager::getInstance();
	
	TimeTrialTrack* ttt = trackList.onEvent(event);
	if (ttt)
	{
		if (*ttt)
		{
			std::vector<Driver*> drivers;
			std::vector<std::string> names;
			std::vector<int> cars;
			
			drivers.push_back(sm.getDriver(0));
			names.push_back(Settings::timetrial.name);
			cars.push_back(Settings::timetrial.car);
			
			sm.enter(new RaceState(ttt->getMap(), RaceSetup(drivers, cars, names, ttt->getLaps(), 1, false, false, new TimeTrialTrack(*ttt))), +1);
		}
		else
		{
			sm.enter(new SelectNewTrack(), +1);
		}
		return;
	}
	
	MenuEvent me = sm.getMenuEvent(event);
	if (me.type == MENU_ESC)
	{
		sm.enter(new TimeTrialMenu(), -1);
		return;
	}
	if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_DELETE)
	{
		TimeTrialTrack* ttt = trackList.getCurrentTrack();
		if (ttt && *ttt)
		{
			sm.enter(new DeleteTrack(*ttt), +1);
			return;
		}
	}
}

void SelectTrack::setStateInfo(int info)
{
	trackList.setState(info);
}

int SelectTrack::getStateInfo()
{
	return trackList.getState();
}
