/*
 * Copyright 2010, 2011, 2012, 2013, 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STATEMANAGER_H
#define STATEMANAGER_H

#include <stack>
#include <vector>
#include <list>
#include "SDL.h"
#include "Rect.h"
#include "Surface.h"
#include "Menu.h"
#include "Settings.h"
#include "Pointer.h"

extern const int GAME_WIDTH;
extern const int GAME_HEIGHT;

class GameState;
class Driver;
class RaceSetup;
class InputDevice;
class Network;
class Scaler;

class StateManager
{
public:
	void cleanUp();
	void run();
	void addUpdateRect(Rect& rect);
	static StateManager& getInstance();
	
	void quit();
	void enter(GameState* state);
	void enter(GameState* state, int);
	
	void updateWholeScreen();
	
	Driver* getDriver(int index);
	InputDevice* getDevice(int index);
	std::vector<std::string> getDeviceNames() const;
	void setDriver(int i, Driver* driver);
	
	MenuEvent getMenuEvent(const SDL_Event& event);
	void forgetLastMenuEvent();
	
	void updateGraphicsSettings(const Settings::Graphics& settings);
	void updateScaler(int newScaler);
	void updateStretch(bool newStretch, bool newKeepAspectRatio);
	void updatePointerType(const std::string& pointerType);
	void updatePointerScaling(int scalePointer);
	
	void setNetwork(Network* net);
	Surface getBackground() const;
	
	const MenuEvent& getLastMenuEvent() const;
	
private:
	StateManager();
	void resize(int width, int height);
	void handleEvent(const SDL_Event& event);
	void updateVideoMode();
	void updateScreen();
	void setNewState(GameState* state);
	void screenToBackgound();
	void savePrevState();
	void toggleFullscreen();
	void forceHideFullscreenPointer();
	
	GameState* currentState;
	Surface screenBuffer;
	bool stateChanged;
	
	std::vector<int> oldStateInfo;
	
	Surface background;

	// graphics stuff
	
	int screenZoom;
	int screenPaddingX;
	int screenPaddingY;
	bool fullscreen;
	// the fullscreen variable is used as an 
	// index to get screenWidth and screenHeight.
	int screenWidth[2];
	int screenHeight[2];
	
	std::vector<SDL_Rect> updateSDLRects;
	std::list<Rect> updateRects;
	std::vector<InputDevice*> devices;
	std::vector<Driver*> drivers;
	
	int xAxis;
	int yAxis;
	
	int bpp;
	Scaler* scaler;
	Network* network;
	
	MenuEvent lastMenuEvent;
	Pointer pointer;
};

char unicode2char(Uint16 unicode);

#endif
