/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "TimeTrialResultState.h"
#include "SurfaceBank.h"
#include "StateManager.h"
#include "Network.h"
#include "RaceState.h"
#include "NetworkMenu.h"
#include "MainMenu.h"
#include "TimeTrialTrack.h"
#include "SelectNewTrack.h"
#include "SelectTrack.h"
#include "str.h"
#include <cmath>

TimeTrialResultState::TimeTrialResultState(const RaceSetup& rs)
:	raceSetup(rs),
	playAgain(false),
	newTrack(false),
	selectTrack(false),
	mainMenu(false)
{
	menu.setTitle("Top times");
	
	menu.addItem("Play again", &playAgain);
	menu.addItem("New track", &newTrack);
	menu.addItem("Select track", &selectTrack);
	menu.addItem("Main menu", &mainMenu);
	
	menu.addRenderObjects(this, CENTER_X, 210);
	
	TimeTrialTrack* ttt = raceSetup.getTimeTrialTrack();
	
	ttt->addTime(raceSetup.getPlayerTime(0));
	ttt->drawTopTimes(background, true);
}

void TimeTrialResultState::onEvent(const SDL_Event& event)
{
	StateManager& sm = StateManager::getInstance();
	MenuEvent me = sm.getMenuEvent(event);
	
	menu.onEvent(me);
	if (playAgain)
	{
		sm.enter(new RaceState(raceSetup.getTimeTrialTrack()->getMap(), raceSetup), 0);
		return;
	}
	else if (newTrack)
	{
		sm.enter(new SelectNewTrack(), 0);
		return;
	}
	else if (selectTrack || me.type == MENU_ESC)
	{
		sm.enter(new SelectTrack(), -1);
		return;
	}
	else if (mainMenu)
	{
		sm.enter(new MainMenu());
		return;
	}
}
