/*
 * Copyright 2014, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TIMETRIALTRACK_H
#define TIMETRIALTRACK_H

#include "Map.h"
#include "Path.h"
#include "Ghost.h"
#include "SDL.h"
#include <vector>
#include <utility>

class Surface;
class GameState;

class TimeTrialTrack
{
public:
	TimeTrialTrack();
	TimeTrialTrack(const Map& map, int laps);
	TimeTrialTrack(Uint32 id);
	void drawTopTimes(Surface& surface, bool compact) const;
	void addTime(int time);
	const Map& getMap() const;
	int getLaps() const;
	static std::vector<TimeTrialTrack> loadFromFile();
	operator bool() const;
	void removeFiles();
	void prepareGhosts(GameState& gameState);
	void ghostUpdate(int x, int y, double rotation);
private:
	void loadGhost(Ghost& ghost, const std::string& name, GameState& gameState);
	
	Uint32 id;
	Map map;
	int laps;
	int personalBest;
	int oldPersonalBest;
	std::size_t newTimeIndex;
	int newTime;
	std::vector<std::pair<std::string, int> > topTimes;
	
	Ghost driverGhost;
	Ghost bestGhost;
	std::vector<GhostState> ghostData;
};

#endif
