/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TRACKLIST_H
#define TRACKLIST_H

#include "TimeTrialTrack.h"
#include "RenderObject.h"
#include "SDL.h"

class GameState;

class TrackList
{
public:
	TrackList(GameState& state);
	TimeTrialTrack* onEvent(const SDL_Event& event);
	TimeTrialTrack* getCurrentTrack();
	void setState(int state);
	int getState();
private:
	struct Track
	{
		TimeTrialTrack timeTrialTrack;
		RenderObject renderObject;
	};
	void updateRenderObject(std::size_t i);
	void updateRenderObjects();
	void updateLeftArrow(bool arrowHover, bool arrowVisible);
	void updateRightArrow(bool arrowHover, bool arrowVisible);
	void next();
	void prev();
	bool scrollRight();
	bool scrollLeft();
	void mouseMove(int x, int y);
	void click(int x, int y, TimeTrialTrack*& ttt);
	
	std::vector<Track> tracks;
	bool selected;
	std::size_t iTrack;
	std::size_t scrollPos;
	GameState& gameState;
	int hoverTrack;
	
	RenderObject leftArrow;
	bool leftArrowHover;
	bool leftArrowVisible;
	
	RenderObject rightArrow;
	bool rightArrowHover;
	bool rightArrowVisible;
	
	RenderObject topTimes;
	
};

#endif
