/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "TrackStateBuffer.h"
#include <algorithm>
#include <cassert>

TrackStateBuffer::TrackStateBuffer()
:	current(0)
{
	states[0].cars = 0;
}

TrackStateBuffer::~TrackStateBuffer()
{
	delete[] states[0].cars;
}

void TrackStateBuffer::setCars(int nCars)
{
	delete[] states[0].cars;
	TrackCarState* cars = new TrackCarState[SIZE * nCars];
	for (std::size_t i = 0; i < SIZE; ++i)
	{
		states[i].cars = cars + nCars * i;
	}
}

void TrackStateBuffer::removeCar(int carId, int nCars)
{
	for (std::size_t i = 0; i < SIZE; ++i)
	{
		TrackCarState* cars = states[i].cars;
		std::copy(cars + carId + 1, cars + nCars, cars + carId);
	}
}

TrackState& TrackStateBuffer::next()
{
	current = (current + 1) % SIZE;
	assert(states[current].cars);
	return states[current];
}

TrackState& TrackStateBuffer::operator[](std::size_t i)
{
	assert(i < SIZE);
	assert(states[i].cars);
	return states[(SIZE + current - i) % SIZE];
}
