/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "StateManager.h"
#include <cstdlib>

const int MIN_RECT_SIZE = 2;
const int MAX_DEPTH = 12;

static void drawPattern(Surface& s, int x, int y, int rectSize, int maxDepth)
{
	if (rectSize <= MIN_RECT_SIZE || maxDepth <= 0)
	{
		return;
	}
	
	SDL_Rect rect;
	rect.w = rectSize;
	rect.h = rectSize;
	rect.x = x;
	rect.y = y;
	Color c = 0;
	switch (std::rand() % 3)
	{
	case 0:
		c = BACKGROUND_COLOR;
		break;
	case 1:
		c = BACKGROUND_COLOR2;
		break;
	case 2:
		c = BACKGROUND_COLOR3;
		break;	
	}
	s.fill(c, &rect);
	
	int newRectSize = rectSize / 2;
	int newMaxDepth = maxDepth - 1 - std::rand() % 2;

	drawPattern(s, x, y, newRectSize, newMaxDepth);
	drawPattern(s, x + newRectSize, y, newRectSize, newMaxDepth);
	drawPattern(s, x, y + newRectSize, newRectSize, newMaxDepth);
	drawPattern(s, x + newRectSize, y + newRectSize, newRectSize, newMaxDepth);
}

Surface createBackground()
{
	int patternSize = GAME_WIDTH + GAME_HEIGHT;
	Surface pattern(patternSize, patternSize);
	
	drawPattern(pattern, 0, 0, 1024, MAX_DEPTH);
	
	Surface background(GAME_WIDTH, GAME_HEIGHT);
	
	for (int y = 0; y < GAME_HEIGHT; ++y)
	{
		for (int x = 0; x < GAME_WIDTH; ++x)
		{
			int sx = x + GAME_HEIGHT / 2;
			int sy = y - GAME_HEIGHT / 2;
			background.setPixel(x, y, pattern.getPixel(sx - sy, sy + sx));
		}
	}
	
	return background;
}
